@@ -1,27 +1,72 @@
 from __future__ import absolute_import
 
 import logging
+import collections
 
 from tornado import web
+from tornado import gen
 
 from ..views import BaseHandler
-from ..models import WorkersModel
 
 
 logger = logging.getLogger(__name__)
 
 
 class ControlHandler(BaseHandler):
-    def is_worker(self, name):
-        return WorkersModel.is_worker(self.application, name)
+    INSPECT_METHODS = ('stats', 'active_queues', 'registered', 'scheduled',
+                       'active', 'reserved', 'revoked', 'conf')
+    worker_cache = collections.defaultdict(dict)
+
+    @gen.coroutine
+    def update_cache(self, workername=None):
+        yield self.update_workers(workername=workername,
+                                  app=self.application)
+
+    @classmethod
+    @gen.coroutine
+    def update_workers(cls, app, workername=None):
+        logger.debug("Updating %s worker's cache...", workername or 'all')
+
+        futures = []
+        destination = [workername] if workername else None
+        timeout = app.options.inspect_timeout / 1000.0
+        inspect = app.capp.control.inspect(
+            timeout=timeout, destination=destination)
+        for method in cls.INSPECT_METHODS:
+            futures.append(app.delay(getattr(inspect, method)))
+
+        results = yield futures
+
+        for i, result in enumerate(results):
+            if result is None:
+                logger.warning("'%s' inspect method failed",
+                               cls.INSPECT_METHODS[i])
+                continue
+            for worker, response in result.items():
+                if response:
+                    info = cls.worker_cache[worker]
+                    info[cls.INSPECT_METHODS[i]] = response
+
+    def is_worker(self, workername):
+        return workername and workername in self.worker_cache
 
     def error_reason(self, workername, response):
         "extracts error message from response"
         for r in response:
             try:
-                return r[workername].get('error', 'Unknown error')
+                return r[workername].get('error', 'Unknown reason')
             except KeyError:
                 pass
+        logger.error("Failed to extract error reason from '%s'", response)
+        return 'Unknown reason'
+
+    @classmethod
+    def get_active_queue_names(cls):
+        queues = set([])
+        for worker, info in cls.worker_cache.items():
+            for q in info.get('active_queues', []):
+                queues.add(q['name'])
+        return queues
 
 
 class WorkerShutDown(ControlHandler):
@@ -57,10 +102,9 @@ Shut down a worker
         """
         if not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
-        celery = self.application.celery_app
 
         logger.info("Shutting down '%s' worker", workername)
-        celery.control.broadcast('shutdown', destination=[workername])
+        self.capp.control.broadcast('shutdown', destination=[workername])
         self.write(dict(message="Shutting down!"))
 
 
@@ -98,13 +142,11 @@ Restart worker's pool
         """
         if not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
-        celery = self.application.celery_app
 
         logger.info("Restarting '%s' worker's pool", workername)
-        response = celery.control.broadcast('pool_restart',
-                                            arguments={'reload': False},
-                                            destination=[workername],
-                                            reply=True)
+        response = self.capp.control.broadcast(
+            'pool_restart', arguments={'reload': False},
+            destination=[workername], reply=True)
         if response and 'ok' in response[0][workername]:
             self.write(dict(
                 message="Restarting '%s' worker's pool" % workername))
@@ -152,20 +194,19 @@ Grow worker's pool
 
         if not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
-        celery = self.application.celery_app
 
         n = self.get_argument('n', default=1, type=int)
 
         logger.info("Growing '%s' worker's pool by '%s'", workername, n)
-        response = celery.control.pool_grow(n=n, reply=True,
-                                            destination=[workername])
+        response = self.capp.control.pool_grow(
+            n=n, reply=True, destination=[workername])
         if response and 'ok' in response[0][workername]:
             self.write(dict(
                 message="Growing '%s' worker's pool by %s" % (workername, n)))
         else:
             logger.error(response)
             self.set_status(403)
-            self.write("Failed to grow '%s' worker's pool" % (
+            self.write("Failed to grow '%s' worker's pool: %s" % (
                 workername, self.error_reason(workername, response)))
 
 
@@ -205,13 +246,12 @@ Shrink worker's pool
 
         if not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
-        celery = self.application.celery_app
 
         n = self.get_argument('n', default=1, type=int)
 
         logger.info("Shrinking '%s' worker's pool by '%s'", workername, n)
-        response = celery.control.pool_shrink(n=n, reply=True,
-                                              destination=[workername])
+        response = self.capp.control.pool_shrink(
+            n=n, reply=True, destination=[workername])
         if response and 'ok' in response[0][workername]:
             self.write(dict(message="Shrinking '%s' worker's pool by %s" % (
                             workername, n)))
@@ -261,17 +301,15 @@ Autoscale worker pool
 
         if not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
-        celery = self.application.celery_app
 
         min = self.get_argument('min', type=int)
         max = self.get_argument('max', type=int)
 
         logger.info("Autoscaling '%s' worker by '%s'",
                     workername, (min, max))
-        response = celery.control.broadcast('autoscale',
-                                            arguments={'min': min, 'max': max},
-                                            destination=[workername],
-                                            reply=True)
+        response = self.capp.control.broadcast(
+            'autoscale', arguments={'min': min, 'max': max},
+            destination=[workername], reply=True)
         if response and 'ok' in response[0][workername]:
             self.write(dict(message="Autoscaling '%s' worker "
                                     "(min=%s, max=%s)" % (
@@ -320,23 +358,21 @@ Start consuming from a queue
         """
         if not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
-        celery = self.application.celery_app
 
         queue = self.get_argument('queue')
 
         logger.info("Adding consumer '%s' to worker '%s'",
                     queue, workername)
-        response = celery.control.broadcast('add_consumer',
-                                            arguments={'queue': queue},
-                                            destination=[workername],
-                                            reply=True)
+        response = self.capp.control.broadcast(
+            'add_consumer', arguments={'queue': queue},
+            destination=[workername], reply=True)
         if response and 'ok' in response[0][workername]:
             self.write(dict(message=response[0][workername]['ok']))
         else:
             logger.error(response)
             self.set_status(403)
             self.write("Failed to add '%s' consumer to '%s' worker: %s" % (
-                workername, self.error_reason(workername, response)
+                queue, workername, self.error_reason(workername, response)
             ))
 
 
@@ -376,16 +412,14 @@ Stop consuming from a queue
         """
         if not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
-        celery = self.application.celery_app
 
         queue = self.get_argument('queue')
 
         logger.info("Canceling consumer '%s' from worker '%s'",
                     queue, workername)
-        response = celery.control.broadcast('cancel_consumer',
-                                            arguments={'queue': queue},
-                                            destination=[workername],
-                                            reply=True)
+        response = self.capp.control.broadcast(
+            'cancel_consumer', arguments={'queue': queue},
+            destination=[workername], reply=True)
         if response and 'ok' in response[0][workername]:
             self.write(dict(message=response[0][workername]['ok']))
         else:
@@ -393,11 +427,11 @@ Stop consuming from a queue
             self.set_status(403)
             self.write(
                 "Failed to cancel '%s' consumer from '%s' worker: %s" % (
-                    workername, self.error_reason(workername, response)
+                    queue, workername, self.error_reason(workername, response)
                 ))
 
 
-class TaskRevoke(BaseHandler):
+class TaskRevoke(ControlHandler):
     @web.authenticated
     def post(self, taskid):
         """
@@ -430,9 +464,8 @@ Revoke a task
 :statuscode 401: unauthorized request
         """
         logger.info("Revoking task '%s'", taskid)
-        celery = self.application.celery_app
         terminate = self.get_argument('terminate', default=False, type=bool)
-        celery.control.revoke(taskid, terminate=terminate)
+        self.capp.control.revoke(taskid, terminate=terminate)
         self.write(dict(message="Revoked '%s'" % taskid))
 
 
@@ -471,13 +504,11 @@ Change soft and hard time limits for a task
 :statuscode 401: unauthorized request
 :statuscode 404: unknown task/worker
         """
-        celery = self.application.celery_app
-
         workername = self.get_argument('workername')
         hard = self.get_argument('hard', default=None, type=float)
         soft = self.get_argument('soft', default=None, type=float)
 
-        if taskname not in celery.tasks:
+        if taskname not in self.capp.tasks:
             raise web.HTTPError(404, "Unknown task '%s'" % taskname)
         if workername is not None and not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
@@ -485,9 +516,9 @@ Change soft and hard time limits for a task
         logger.info("Setting timeouts for '%s' task (%s, %s)",
                     taskname, soft, hard)
         destination = [workername] if workername is not None else None
-        response = celery.control.time_limit(taskname, reply=True,
-                                             hard=hard, soft=soft,
-                                             destination=destination)
+        response = self.capp.control.time_limit(
+            taskname, reply=True, hard=hard, soft=soft,
+            destination=destination)
 
         if response and 'ok' in response[0][workername]:
             self.write(dict(message=response[0][workername]['ok']))
@@ -527,18 +558,16 @@ Change rate limit for a task
       "message": "Revoked '1480b55c-b8b2-462c-985e-24af3e9158f9'"
   }
 
-:query terminate: terminate the task if it is running
+:query workername: worker name
 :reqheader Authorization: optional OAuth token to authenticate
 :statuscode 200: no error
 :statuscode 401: unauthorized request
 :statuscode 404: unknown task/worker
         """
-        celery = self.application.celery_app
-
         workername = self.get_argument('workername')
         ratelimit = self.get_argument('ratelimit')
 
-        if taskname not in celery.tasks:
+        if taskname not in self.capp.tasks:
             raise web.HTTPError(404, "Unknown task '%s'" % taskname)
         if workername is not None and not self.is_worker(workername):
             raise web.HTTPError(404, "Unknown worker '%s'" % workername)
@@ -546,10 +575,8 @@ Change rate limit for a task
         logger.info("Setting '%s' rate limit for '%s' task",
                     ratelimit, taskname)
         destination = [workername] if workername is not None else None
-        response = celery.control.rate_limit(taskname,
-                                             ratelimit,
-                                             reply=True,
-                                             destination=destination)
+        response = self.capp.control.rate_limit(
+            taskname, ratelimit, reply=True, destination=destination)
         if response and 'ok' in response[0][workername]:
             self.write(dict(message=response[0][workername]['ok']))
         else:
