@@ -3,6 +3,8 @@ from __future__ import absolute_import
 import json
 import logging
 
+from datetime import datetime
+
 from tornado import web
 from tornado.escape import json_decode
 from tornado.web import HTTPError
@@ -11,7 +13,7 @@ from celery import states
 from celery.result import AsyncResult
 from celery.backends.base import DisabledBackend
 
-from ..models import TaskModel
+from ..utils import tasks
 from ..views import BaseHandler
 
 
@@ -51,6 +53,8 @@ class BaseTaskHandler(BaseHandler):
 
 
 class TaskAsyncApply(BaseTaskHandler):
+    DATE_FORMAT = '%Y-%m-%d %H:%M:%S.%f'
+
     @web.authenticated
     def post(self, taskname):
         """
@@ -87,28 +91,46 @@ Execute a task
 
 :query args: a list of arguments
 :query kwargs: a dictionary of arguments
+:query options: a dictionary of `apply_async` keyword arguments
 :reqheader Authorization: optional OAuth token to authenticate
 :statuscode 200: no error
 :statuscode 401: unauthorized request
 :statuscode 404: unknown task
         """
-        celery = self.application.celery_app
-
         args, kwargs, options = self.get_task_args()
         logger.info("Invoking a task '%s' with '%s' and '%s'",
                     taskname, args, kwargs)
 
         try:
-            task = celery.tasks[taskname]
+            task = self.capp.tasks[taskname]
         except KeyError:
             raise HTTPError(404, "Unknown task '%s'" % taskname)
 
+        try:
+            self.normalize_options(options)
+        except ValueError:
+            raise HTTPError(400, 'Invalid option')
+
         result = task.apply_async(args=args, kwargs=kwargs, **options)
         response = {'task-id': result.task_id}
         if self.backend_configured(result):
             response.update(state=result.state)
         self.write(response)
 
+    def normalize_options(self, options):
+        if 'eta' in options:
+            options['eta'] = datetime.strptime(options['eta'],
+                                               self.DATE_FORMAT)
+        if 'countdown' in options:
+            options['countdown'] = float(options['countdown'])
+        if 'expires' in options:
+            expires = options['expires']
+            try:
+                expires = float(expires)
+            except ValueError:
+                expires = datetime.strptime(expires, self.DATE_FORMAT)
+            options['expires'] = expires
+
 
 class TaskSend(BaseTaskHandler):
     @web.authenticated
@@ -151,13 +173,11 @@ Execute a task by name (doesn't require task sources)
 :statuscode 401: unauthorized request
 :statuscode 404: unknown task
         """
-        celery = self.application.celery_app
-
         args, kwargs, options = self.get_task_args()
         logger.debug("Invoking task '%s' with '%s' and '%s'",
                      taskname, args, kwargs)
-        result = celery.send_task(taskname, args=args,
-                                  kwargs=kwargs, **options)
+        result = self.capp.send_task(
+            taskname, args=args, kwargs=kwargs, **options)
         response = {'task-id': result.task_id}
         if self.backend_configured(result):
             response.update(state=result.state)
@@ -307,17 +327,57 @@ List tasks
         type = type if type != 'All' else None
         state = state if state != 'All' else None
 
-        tasks = []
-        for task_id, task in TaskModel.iter_tasks(
-                app, limit=limit, type=type,
+        result = []
+        for task_id, task in tasks.iter_tasks(
+                app.events, limit=limit, type=type,
                 worker=worker, state=state):
             task = task.as_dict()
             task.pop('worker')
-            tasks.append((task_id, task))
-        self.write(dict(tasks))
+            result.append((task_id, task))
+        self.write(dict(result))
+
+
+class ListTaskTypes(BaseTaskHandler):
+    @web.authenticated
+    def get(self):
+        """
+List (seen) task types
+
+**Example request**:
+
+.. sourcecode:: http
+
+  GET /api/task/types HTTP/1.1
+  Host: localhost:5555
+
+**Example response**:
+
+.. sourcecode:: http
+
+  HTTP/1.1 200 OK
+  Content-Length: 44
+  Content-Type: application/json; charset=UTF-8
+
+  {
+      "task-types": [
+          "tasks.add",
+          "tasks.sleep"
+      ]
+  }
+
+:reqheader Authorization: optional OAuth token to authenticate
+:statuscode 200: no error
+:statuscode 401: unauthorized request
+        """
+        seen_task_types = self.application.events.state.task_types()
+
+        response = {}
+        response['task-types'] = seen_task_types
+        self.write(response)
 
 
 class TaskInfo(BaseTaskHandler):
+    @web.authenticated
     def get(self, taskid):
         """
 Get a task info
@@ -352,7 +412,7 @@ Get a task info
       "kwargs": "{}",
       "name": "tasks.add",
       "received": 1400806241.970742,
-      "result": "'{\"result\": 4}'",
+      "result": "'4'",
       "retried": null,
       "retries": null,
       "revoked": null,
@@ -374,7 +434,7 @@ Get a task info
 :statuscode 404: unknown task
         """
 
-        task = TaskModel.get_task_by_id(self.application, taskid)
+        task = tasks.get_task_by_id(self.application.events, taskid)
         if not task:
             raise HTTPError(404, "Unknown task '%s'" % taskid)
         response = {}
