@@ -36,8 +36,10 @@ var flower = (function () {
     function toggle_selected_workers(event) {
         var $checkbox = $('#select-workers-toggler');
 
-        $checkbox.is(':checked') ? select_all_workers()
-                                 : select_none_workers();
+        if ($checkbox.is(':checked'))
+            select_all_workers();
+        else
+            select_none_workers();
     }
 
     function shutdown_selected(event) {
@@ -50,7 +52,7 @@ var flower = (function () {
 
             $.ajax({
                 type: 'POST',
-                url: url_prefix() + '/api/worker/shutdown/' + worker_name,
+                url: '/api/worker/shutdown/' + worker_name,
                 dataType: 'json',
                 data: { workername: worker_name },
                 success: function (data) {
@@ -73,7 +75,7 @@ var flower = (function () {
 
             $.ajax({
                 type: 'POST',
-                url: url_prefix() + '/api/worker/pool/restart/' + worker_name,
+                url: '/api/worker/pool/restart/' + worker_name,
                 dataType: 'json',
                 data: { workername: worker_name },
                 success: function (data) {
@@ -86,6 +88,60 @@ var flower = (function () {
         });
     }
 
+    function refresh_selected(event) {
+        var $selected_workers = get_selected_workers();
+
+        if (!$selected_workers.length) {
+            $.ajax({
+                type: 'GET',
+                url: '/api/workers',
+                data: { refresh: 1 },
+                success: function (data) {
+                    show_success_alert('Refreshed');
+                },
+                error: function (data) {
+                    show_error_alert(data.responseText);
+                }
+            });
+        }
+
+        $selected_workers.each(function () {
+            var $worker = $(this),
+                worker_name = $worker.attr('id');
+
+            $.ajax({
+                type: 'GET',
+                url: '/api/workers',
+                dataType: 'json',
+                data: { workername: unescape(worker_name), refresh: 1 },
+                success: function (data) {
+                    show_success_alert(data.message || 'Refreshed');
+                },
+                error: function (data) {
+                    show_error_alert(data.responseText);
+                }
+            });
+        });
+    }
+
+    function on_worker_refresh(event) {
+        event.preventDefault();
+        event.stopPropagation();
+
+        $.ajax({
+            type: 'GET',
+            url: window.location.pathname,
+            data: 'refresh=1',
+            success: function (data) {
+                //show_success_alert('Refreshed');
+                window.location.reload();
+            },
+            error: function (data) {
+                show_error_alert(data.responseText);
+            }
+        });
+    }
+
     function on_pool_grow(event) {
         event.preventDefault();
         event.stopPropagation();
@@ -95,7 +151,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/worker/pool/grow/' + workername,
+            url: '/api/worker/pool/grow/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -119,7 +175,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/worker/pool/shrink/' + workername,
+            url: '/api/worker/pool/shrink/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -144,7 +200,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/worker/pool/autoscale/' + workername,
+            url: '/api/worker/pool/autoscale/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -169,7 +225,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/worker/queue/add-consumer/' + workername,
+            url: '/api/worker/queue/add-consumer/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -178,7 +234,7 @@ var flower = (function () {
             success: function (data) {
                 show_success_alert(data.message);
                 setTimeout(function () {
-                    $('#tab-queues').load(url_prefix() + '/worker/' + workername + ' #tab-queues').fadeIn('show');
+                    $('#tab-queues').load('/worker/' + workername + ' #tab-queues').fadeIn('show');
                 }, 10000);
             },
             error: function (data) {
@@ -196,7 +252,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/worker/queue/cancel-consumer/' + workername,
+            url: '/api/worker/queue/cancel-consumer/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -205,7 +261,7 @@ var flower = (function () {
             success: function (data) {
                 show_success_alert(data.message);
                 setTimeout(function () {
-                    $('#tab-queues').load(url_prefix() + '/worker/' + workername + ' #tab-queues').fadeIn('show');
+                    $('#tab-queues').load('/worker/' + workername + ' #tab-queues').fadeIn('show');
                 }, 10000);
             },
             error: function (data) {
@@ -221,21 +277,18 @@ var flower = (function () {
         var workername = $('#workername').text(),
             taskname = $(event.target).closest("tr").children("td:eq(0)").text(),
             type = $(event.target).html().toLowerCase(),
-            timeout = $(event.target).siblings().closest("input").val(),
-            data = {};
+            timeout = $(event.target).siblings().closest("input").val();
 
         taskname = taskname.split(' ')[0]; // removes [rate_limit=xxx]
 
-        console.log(type);
-
-        data.taskname = taskname;
-        data[type] = timeout;
-
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/task/timeout/' + workername,
+            url: url_prefix() + '/api/task/timeout/' + taskname,
             dataType: 'json',
-            data: data,
+            data: {
+                'workername': workername,
+                'type': timeout,
+            },
             success: function (data) {
                 show_success_alert(data.message);
             },
@@ -257,16 +310,16 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/task/rate-limit/' + workername,
+            url: '/api/task/rate-limit/' + taskname,
             dataType: 'json',
             data: {
-                'taskname': taskname,
+                'workername': workername,
                 'ratelimit': ratelimit,
             },
             success: function (data) {
                 show_success_alert(data.message);
                 setTimeout(function () {
-                    $('#tab-limits').load(url_prefix() + '/worker/' + workername + ' #tab-limits').fadeIn('show');
+                    $('#tab-limits').load('/worker/' + workername + ' #tab-limits').fadeIn('show');
                 }, 10000);
             },
             error: function (data) {
@@ -283,7 +336,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/task/revoke/' + taskid,
+            url: '/api/task/revoke/' + taskid,
             dataType: 'json',
             data: {
                 'terminate': false,
@@ -305,7 +358,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: url_prefix() + '/api/task/revoke/' + taskid,
+            url: '/api/task/revoke/' + taskid,
             dataType: 'json',
             data: {
                 'terminate': true,
@@ -319,7 +372,10 @@ var flower = (function () {
         });
     }
 
-    function on_workers_table_update(update) {
+    function on_dashboard_update(update) {
+        var total_active = 0, total_processed = 0, total_failed = 0,
+            total_succeeded = 0, total_retried = 0;
+
         $.each(update, function (name) {
             var id = encodeURIComponent(name),
                 sel = id.replace(/([ #;&,.+*~\':"!^$[\]()=>|\/%@])/g,'\\$1'),
@@ -328,23 +384,40 @@ var flower = (function () {
             if (tr.length === 0) {
                 $('#workers-table-row').clone().removeClass('hidden').attr('id', id).appendTo('tbody');
                 tr = $('#' + sel);
-                tr.children('td').children('a').attr('href', url_prefix() + '/worker/' + name).text(name);
+                tr.children('td').children('a').attr('href', '/worker/' + name).text(name);
             }
 
             var stat = tr.children('td:eq(2)').children(),
-                concurrency = tr.children('td:eq(3)'),
-                completed_tasks = tr.children('td:eq(4)'),
-                running_tasks = tr.children('td:eq(5)'),
-                queues = tr.children('td:eq(6)');
+                active = tr.children('td:eq(3)'),
+                processed = tr.children('td:eq(4)'),
+                failed = tr.children('td:eq(5)'),
+                succeeded = tr.children('td:eq(6)'),
+                retried = tr.children('td:eq(7)'),
+                loadavg = tr.children('td:eq(8)');
 
             stat.text($(this).attr('status') ? "Online" : "Offline");
             stat.removeClass("label-success label-important");
             stat.addClass($(this).attr('status') ? "label-success" : "label-important");
-            concurrency.text($(this).attr('concurrency'));
-            completed_tasks.text($(this).attr('completed_tasks'));
-            running_tasks.text($(this).attr('running_tasks'));
-            queues.text($(this).attr('queues').toString().replace(/,/g, ', '));
+            active.text($(this).attr('active'));
+            processed.text($(this).attr('processed'));
+            failed.text($(this).attr('failed'));
+            succeeded.text($(this).attr('succeeded'));
+            retried.text($(this).attr('retried'));
+            loadavg.text($(this).attr('loadavg').toString().replace(/,/g, ', '));
+
+            total_active += $(this).attr('active') ? $(this).attr('active') > 0 : 0;
+            total_processed += $(this).attr('processed');
+            total_failed += $(this).attr('failed');
+            total_succeeded += $(this).attr('succeeded');
+            total_retried += $(this).attr('retried');
+
         });
+
+        $('a#btn-active').text('Active: ' + total_active);
+        $('a#btn-processed').text('Processed: ' + total_processed);
+        $('a#btn-failed').text('Failed: ' + total_failed);
+        $('a#btn-succeeded').text('Succeeded: ' + total_succeeded);
+        $('a#btn-retried').text('Retried: ' + total_retried);
     }
 
 
@@ -354,6 +427,7 @@ var flower = (function () {
 
         $('#task-filter-form').each(function () {
             $(this).find('SELECT').val('');
+            $(this).find('.datetimepicker').val('');
         });
 
         $('#task-filter-form').submit();
@@ -361,8 +435,8 @@ var flower = (function () {
 
     function create_graph(data, id, width, height) {
         id = id || '';
-        width = width || 700;
-        height = height || 400;
+        width = width || 500;
+        height = height || 300;
 
         var name, seriesData = [];
         for (name in data) {
@@ -378,13 +452,20 @@ var flower = (function () {
             renderer: 'stack',
             series: new Rickshaw.Series(seriesData, palette),
             maxDataPoints: 10000,
+            padding: {top: 0.1, left: 0.01, right: 0.01, bottom: 0.01},
         });
 
         var ticksTreatment = 'glow';
 
+        var timeUnit = new Rickshaw.Fixtures.Time.Local();
+        timeUnit.formatTime = function(d) { return moment(d).format("yyyy.mm.dd HH:mm:ss"); };
+        timeUnit.unit("minute");
+
         var xAxis = new Rickshaw.Graph.Axis.Time({
             graph: graph,
+            timeFixture: new Rickshaw.Fixtures.Time.Local(),
             ticksTreatment: ticksTreatment,
+            timeUnit: timeUnit
         });
 
         xAxis.render();
@@ -392,18 +473,19 @@ var flower = (function () {
         var yAxis = new Rickshaw.Graph.Axis.Y({
             graph: graph,
             tickFormat: Rickshaw.Fixtures.Number.formatKMBT,
-            ticksTreatment: ticksTreatment
+            ticksTreatment: ticksTreatment,
         });
 
         yAxis.render();
 
-        var slider = new Rickshaw.Graph.RangeSlider({
-            graph: graph,
-            element: $('#slider' + id)
-        });
-
         var hoverDetail = new Rickshaw.Graph.HoverDetail({
-            graph: graph
+            graph: graph,
+            yFormatter: function(y) {
+                if (y % 1 === 0)
+                    return y;
+                else
+                    return y.toFixed(2);
+            }
         });
 
         var legend = new Rickshaw.Graph.Legend({
@@ -452,32 +534,24 @@ var flower = (function () {
                         now.getUTCMinutes(), now.getUTCSeconds())/1000;
     }
 
-    function url_prefix() {
-        // prefix is initialized in base.html
-        if (prefix) {
-            return '/' + prefix;
-        }
-        return '';
-    }
-
     $.urlParam = function(name){
         var results = new RegExp('[\\?&]' + name + '=([^&#]*)').exec(window.location.href);
         return results && results[1] || 0;
-    }
+    };
 
     $(document).ready(function () {
-        if ($.inArray($(location).attr('pathname'), [url_prefix(), url_prefix() + '/workers'])) {
+        if ($.inArray($(location).attr('pathname'), ['/', '/dashboard']) != -1) {
             var host = $(location).attr('host'),
                 protocol = $(location).attr('protocol') == 'http:' ? 'ws://' : 'wss://',
-                ws = new WebSocket(protocol + host + url_prefix() + "/update-workers");
+                ws = new WebSocket(protocol + host + "/update-dashboard");
             ws.onmessage = function (event) {
                 var update = $.parseJSON(event.data);
-                on_workers_table_update(update);
+                on_dashboard_update(update);
             };
         }
 
         //https://github.com/twitter/bootstrap/issues/1768
-        var shiftWindow = function() { scrollBy(0, -50) };
+        var shiftWindow = function() { scrollBy(0, -50); };
         if (location.hash) shiftWindow();
         window.addEventListener("hashchange", shiftWindow);
 
@@ -492,7 +566,7 @@ var flower = (function () {
             });
         });
 
-        if ($(location).attr('pathname') === url_prefix() + '/monitor') {
+        if ($(location).attr('pathname') === '/monitor') {
             var sts = current_unix_time(),
                 fts = current_unix_time(),
                 tts = current_unix_time(),
@@ -504,7 +578,7 @@ var flower = (function () {
 
             $.ajax({
                 type: 'GET',
-                url: url_prefix() + '/monitor/succeeded-tasks',
+                url: '/monitor/succeeded-tasks',
                 data: {lastquery: current_unix_time()},
                 success: function (data) {
                     succeeded_graph = create_graph(data, '-succeeded');
@@ -513,7 +587,7 @@ var flower = (function () {
                     succeeded_graph.series.setTimeInterval(updateinterval);
                     setInterval(function () {
                         update_graph(succeeded_graph,
-                                     url_prefix() + '/monitor/succeeded-tasks',
+                                     '/monitor/succeeded-tasks',
                                      sts);
                         sts = current_unix_time();
                     }, updateinterval);
@@ -523,7 +597,7 @@ var flower = (function () {
 
             $.ajax({
                 type: 'GET',
-                url: url_prefix() + '/monitor/completion-time',
+                url: '/monitor/completion-time',
                 data: {lastquery: current_unix_time()},
                 success: function (data) {
                     time_graph = create_graph(data, '-time');
@@ -532,7 +606,7 @@ var flower = (function () {
                     time_graph.series.setTimeInterval(updateinterval);
                     setInterval(function () {
                         update_graph(time_graph,
-                                     url_prefix() + '/monitor/completion-time',
+                                     '/monitor/completion-time',
                                      tts);
                         tts = current_unix_time();
                     }, updateinterval);
@@ -542,7 +616,7 @@ var flower = (function () {
 
             $.ajax({
                 type: 'GET',
-                url: url_prefix() + '/monitor/failed-tasks',
+                url: '/monitor/failed-tasks',
                 data: {lastquery: current_unix_time()},
                 success: function (data) {
                     failed_graph = create_graph(data, '-failed');
@@ -551,7 +625,7 @@ var flower = (function () {
                     failed_graph.series.setTimeInterval(updateinterval);
                     setInterval(function () {
                         update_graph(failed_graph,
-                                     url_prefix() + '/monitor/failed-tasks',
+                                     '/monitor/failed-tasks',
                                      fts);
                         fts = current_unix_time();
                     }, updateinterval);
@@ -561,7 +635,7 @@ var flower = (function () {
 
             $.ajax({
                 type: 'GET',
-                url: url_prefix() + '/monitor/broker',
+                url: '/monitor/broker',
                 success: function (data) {
                     broker_graph = create_graph(data, '-broker');
                     broker_graph.update();
@@ -569,7 +643,7 @@ var flower = (function () {
                     broker_graph.series.setTimeInterval(updateinterval);
                     setInterval(function () {
                         update_graph(broker_graph,
-                                     url_prefix() + '/monitor/broker');
+                                     '/monitor/broker');
                     }, updateinterval);
 
                 },
@@ -585,7 +659,9 @@ var flower = (function () {
         select_none_workers: select_none_workers,
         shutdown_selected: shutdown_selected,
         restart_selected: restart_selected,
+        refresh_selected: refresh_selected,
         on_alert_close: on_alert_close,
+        on_worker_refresh: on_worker_refresh,
         on_pool_grow: on_pool_grow,
         on_pool_shrink: on_pool_shrink,
         on_pool_autoscale: on_pool_autoscale,
