@@ -1,49 +1,35 @@
 from __future__ import absolute_import
 
+import os
+
 from tornado.web import StaticFileHandler
 
-from .views.workers import (
-    WorkersView,
-    WorkerView,
-)
-
-from .views.tasks import (
-    TaskView,
-    TasksView,
-)
-
-from .views.broker import (
-    BrokerView,
-)
-
-from .views import auth
-
 from .api import events
 from .api import control
 from .api import tasks
 from .api import workers
-
-from .views.update import (
-    UpdateWorkers,
-)
-
-from .views.monitor import (
-    Monitor,
-    SucceededTaskMonitor,
-    FailedTaskMonitor,
-    TimeToCompletionMonitor,
-    BrokerMonitor,
-)
-
-
+from .views import auth
+from .views import monitor
+from .views.broker import BrokerView
+from .views.workers import WorkerView
+from .views.tasks import TaskView, TasksView
 from .views.error import NotFoundErrorHandler
-from .settings import APP_SETTINGS
+from .views.dashboard import DashboardView, DashboardUpdateHandler
+from .utils import gen_cookie_secret
+
+
+settings = dict(
+    template_path=os.path.join(os.path.dirname(__file__), "templates"),
+    static_path=os.path.join(os.path.dirname(__file__), "static"),
+    cookie_secret=gen_cookie_secret(),
+    login_url='/login',
+)
 
 
 handlers = [
     # App
-    (r"/", WorkersView),
-    (r"/workers", WorkersView),
+    (r"/", DashboardView),
+    (r"/dashboard", DashboardView),
     (r"/worker/(.+)", WorkerView),
     (r"/task/(.+)", TaskView),
     (r"/tasks", TasksView),
@@ -60,6 +46,7 @@ handlers = [
         control.WorkerQueueCancelConsumer),
     # Task API
     (r"/api/tasks", tasks.ListTasks),
+    (r"/api/task/types", tasks.ListTaskTypes),
     (r"/api/task/info/(.*)", tasks.TaskInfo),
     (r"/api/task/async-apply/(.+)", tasks.TaskAsyncApply),
     (r"/api/task/send-task/(.+)", tasks.TaskSend),
@@ -76,16 +63,16 @@ handlers = [
     (r"/api/task/events/task-revoked/(.*)", events.TaskRevoked),
     (r"/api/task/events/task-retried/(.*)", events.TaskRetried),
     # WebSocket Updates
-    (r"/update-workers", UpdateWorkers),
+    (r"/update-dashboard", DashboardUpdateHandler),
     # Monitors
-    (r"/monitor", Monitor),
-    (r"/monitor/succeeded-tasks", SucceededTaskMonitor),
-    (r"/monitor/failed-tasks", FailedTaskMonitor),
-    (r"/monitor/completion-time", TimeToCompletionMonitor),
-    (r"/monitor/broker", BrokerMonitor),
+    (r"/monitor", monitor.Monitor),
+    (r"/monitor/succeeded-tasks", monitor.SucceededTaskMonitor),
+    (r"/monitor/failed-tasks", monitor.FailedTaskMonitor),
+    (r"/monitor/completion-time", monitor.TimeToCompletionMonitor),
+    (r"/monitor/broker", monitor.BrokerMonitor),
     # Static
     (r"/static/(.*)", StaticFileHandler,
-     {"path": APP_SETTINGS['static_path']}),
+     {"path": settings['static_path']}),
     # Auth
     (r"/login", auth.LoginHandler),
     (r"/logout", auth.LogoutHandler),
